<?php

namespace Drupal\webform_select_collection\Plugin\WebformElement;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\OptGroup;
use Drupal\webform\Entity\WebformOptions;
use Drupal\webform\Plugin\WebformElement\OptionsBase;
use Drupal\webform\Plugin\WebformElement\WebformTableTrait;
use Drupal\webform\Utility\WebformOptionsHelper;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Provides a 'webform_select_collection' element.
 *
 * @WebformElement(
 *   id = "webform_select_collection",
 *   label = @Translation("Select collection"),
 *   description = @Translation("Provides a form element for a table with selects or checkboxes in right column."),
 *   category = @Translation("Options elements"),
 *   states_wrapper = TRUE,
 *   composite = TRUE,
 * )
 */
class WebformSelectCollection extends OptionsBase {

  use WebformTableTrait;

  /**
   * {@inheritdoc}
   */
  public function getDefaultProperties() {
    $properties = [
      // Table settings.
      'line_items' => [],
      'line_options' => [],
      'js_select' => TRUE,
      // @todo Handle checkboxes
    ] + parent::getDefaultProperties();

    unset($properties['options']);
    unset($properties['options_randomize']);

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function supportsMultipleValues() {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getElementSelectorOptions(array $element) {
    $title = $this->getAdminLabel($element) . ' [' . $this->getPluginLabel() . ']';
    $name = $element['#webform_key'];
    // @todo Handle checkboxes
    $type = ($this->hasMultipleValues($element) ? $this->t('Checkbox') : $this->t('Select'));

    $selectors = [];
    foreach ($element['#line_options'] as $value => $text) {
      if (is_array($text)) {
        $text = $value;
      }
      $selectors[":input[name=\"{$name}[{$value}]\"]"] = $text . ' [' . $type . ']';
    }
    return [$title => $selectors];
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    unset($form['options']);

    $form['line_items'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Element line items'),
      '#open' => TRUE,
    ];
    $form['line_items']['line_items'] = [
      '#type' => 'webform_element_options',
      '#title' => $this->t('Line items'),
      '#required' => TRUE,
      '#options_description' => FALSE,
    ];

    $form['line_items']['js_select'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Select all'),
      '#description' => $this->t('If checked, a select all select/checkbox will be added to the header.'),
      '#return_value' => TRUE,
    ];

    $form['line_options'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Element line item options'),
      '#open' => TRUE,
    ];
    $form['line_options']['line_options'] = [
      '#type' => 'webform_element_options',
      '#title' => $this->t('Item options'),
      '#required' => TRUE,
      '#options_description' => FALSE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function prepare(array &$element, WebformSubmissionInterface $webform_submission = NULL) {
    parent::prepare($element, $webform_submission);

    // Add missing element class.
    $element['#attributes']['class'][] = str_replace('_', '-', $element['#type']);
  }

  /**
   * {@inheritdoc}
   */
  protected function formatHtmlItem(array $element, WebformSubmissionInterface $webform_submission, array $options = []) {
    $list = [
      '#theme' => 'item_list',
      '#items' => [],
      '#list_type' => 'ul',
    ];

    $value = $this->getValue($element, $webform_submission, $options);
    $format = $this->getItemFormat($element);
    $format_raw = ($format === 'raw');

    if (!$format_raw) {
      $flattened_options = OptGroup::flattenOptions($element['#line_options']);
      $flattened_items = OptGroup::flattenOptions($element['#line_items']);
    }

    foreach ($value as $item => $property) {
      if ($format_raw) {
        $placeholders = [
          '@item' => $item,
          '@property' => $property,
        ];
      }
      else {
        $placeholders = [
          '@item' => WebformOptionsHelper::getOptionText($item, $flattened_items),
          '@property' => WebformOptionsHelper::getOptionText($property, $flattened_options),
        ];
      }
      $list['#items'][] = $this->t('@item:&nbsp;@property', $placeholders);
    }

    return $list;
  }

  /**
   * {@inheritdoc}
   */
  protected function formatTextItem(array $element, WebformSubmissionInterface $webform_submission, array $options = []) {
    $text = '';
    $items = [];
    $delimiter = empty($options['multiple_delimiter']) ? '; ' : $options['multiple_delimiter'];

    $value = $this->getValue($element, $webform_submission, $options);
    $format = $this->getItemFormat($element);

    if ($format === 'raw') {
      foreach ($value as $item => $property) {
        $items[] = $item . ':' . $property;
      }
      $text = implode($delimiter, $items);
      return $text;
    }

    if (isset($element['#line_options'])) {
      $flattened_options = OptGroup::flattenOptions($element['#line_options']);
      $flattened_items = OptGroup::flattenOptions($element['#line_items']);
      foreach ($value as $item => $property) {
        $items[] = WebformOptionsHelper::getOptionText($item, $flattened_items)
         . ': '
         . WebformOptionsHelper::getOptionText($property, $flattened_options);
      }
      $text = implode($delimiter, $items);
    }

    $options += ['prefixing' => TRUE];
    if ($options['prefixing']) {
      if (isset($element['#field_prefix'])) {
        $text = strip_tags($element['#field_prefix']) . $text;
      }
      if (isset($element['#field_suffix'])) {
        $text .= strip_tags($element['#field_suffix']);
      }
    }

    return $text;
  }

  /**
   * {@inheritdoc}
   */
  public function initialize(array &$element) {
    parent::initialize($element);
    // Set element items.
    if (isset($element['#line_items'])) {
      $element['#line_items'] = WebformOptions::getElementOptions($element, '#line_items');
    }
    // Set element options.
    if (isset($element['#line_options'])) {
      $element['#line_options'] = WebformOptions::getElementOptions($element, '#line_options');
    }
  }

}
